# Kotlin Design Patterns and Best Practices - Second Edition

<a href="https://www.packtpub.com/product/kotlin-design-patterns-and-best-practices-second-edition/9781801815727?utm_source=github&utm_medium=repository&utm_campaign=9781801815727"><img src="https://static.packt-cdn.com/products/9781801815727/cover/smaller" alt="About the Author" height="256px" align="right"></a>

This is the code repository for [Kotlin Design Patterns and Best Practices - Second Edition](https://www.packtpub.com/product/kotlin-design-patterns-and-best-practices-second-edition/9781801815727?utm_source=github&utm_medium=repository&utm_campaign=9781801815727), published by Packt.

**Build scalable applications using traditional, reactive, and concurrent design patterns in Kotlin**

## What is this book about?
Design patterns are well-known solutions to common architectural problems as they allow you to solve many problems efficiently and create a shared vocabulary between developers. This book shows you how easy it can be to implement traditional design patterns in the modern multi-paradigm Kotlin programming language, and takes you through the new patterns and paradigms that have emerged. 

This book covers the following exciting features:
- Implement all the classical design patterns using the Kotlin programming language.
- Apply reactive and concurrent design patterns to make your application more scalable.
- Discover best practices in Kotlin and explore its new features.
- Understand the key principles of functional programming and learn how they apply to Kotlin.
- Find out how to write idiomatic Kotlin code and learn which patterns to avoid.
- Harness the power of Kotlin to design concurrent and reliable systems with ease.
- Create an effective microservice with Kotlin and the Ktor framework.

If you feel this book is for you, get your [copy](https://www.amazon.com/dp/1801815720) today!

## Instructions and Navigations
All of the code is organized into folders. For example, Chapter02.

The code will look like the following:
```
val chan = produce(capacity = 10) {
    (1..10).forEach {
        send(it)
    }
}
```

**Following is what you need for this book:**
This book is for developers looking to apply design patterns they've learned from other languages in Kotlin to build reliable, scalable, and maintainable applications. Prior programming knowledge is necessary to get started with this book. Experience in Java or design patterns is helpful, but not mandatory.

With the following software and hardware list you can run all code files present in the book (Chapter 1-11).
### Software and Hardware List
| Chapter | Software required | OS required |
| -------- | ------------------------------------ | ----------------------------------- |
| 1-11 | Kotlin, IntelliJ IDEA Community Edition, OpenJDK 11 or higher  | Windows, Mac OS X, and Linux (Any) |


We also provide a PDF file that has color images of the screenshots/diagrams used in this book. [Click here to download it](https://static.packt-cdn.com/downloads/9781801815727_ColorImages.pdf).

### Related products
* How to Build Android Apps with Kotlin [[Packt]](https://www.packtpub.com/free-ebook/how-to-build-android-apps-with-kotlin/9781838984113?utm_source=github&utm_medium=repository&utm_campaign=9781838984113) [[Amazon]](https://www.amazon.com/dp/1838984119)

* Mastering Kotlin [[Packt]](https://www.packtpub.com/product/mastering-kotlin/9781838555726?utm_source=github&utm_medium=repository&utm_campaign=9781838555726) [[Amazon]](https://www.amazon.com/dp/1838555722)


## Get to Know the Author
**Alexey Soshin**
is a software architect with 15 years of experience in the industry. He started exploring Kotlin when Kotlin was still in beta, and since then has been a big enthusiast of the language. He's a conference speaker, published writer, and the author of a video course titled Pragmatic System Design.

## Other books by the authors
* [Hands-on Design Patterns with Kotlin](https://www.packtpub.com/product/hands-on-design-patterns-with-kotlin/9781788998017?utm_source=github&utm_medium=repository&utm_campaign=9781788998017)
